/**********************************************************************************************

  DSP Solutions GmbH & Co. KG
  http://www.dspsolutions.de/

  Author:      Not defined
  Email:       Not defined
  Phone:       Not defined
  Fax:         +49-941-83055-79

  File:        IItem.h

  Module:

  Description:

  Created:     09/04/2010

  (C) 2010 DSP Solutions. All rights reserved.


**********************************************************************************************/
#ifndef IITEM_H
#define IITEM_H

#include <QObject>
#include <QPixmap>

/// interface class for all geo items to unify basic data and access
class IItem : public QObject
{
    Q_OBJECT;
    public:
        IItem(QObject * parent);
        virtual ~IItem();

        /// set the item's short name
        virtual void setName(const QString& str){name = str;}
        /// get the item's short name
        virtual QString getName(){return name;}

        /// set the item's long comment
        virtual void setComment(const QString& str){comment = str;}
        /// get the item's long comment
        virtual QString getComment(){return comment;}

        /// get a summary of item's data to display on screen or in the toolview
        virtual QString getInfo()= 0;

        /// set the icon defined by a string
        virtual void setIcon(const QString& str) = 0;
        /// get the icon as pixmap
        virtual QPixmap getIcon(){return iconPixmap;}
        /// get the icon as string definition
        virtual QString getIconString(){return iconString;}

        /// set the internal unique key
        /**
            CAUTION! Use it wisely. Usually the key is generated by the item.
        */
        virtual void setKey(const QString& str){key = str;}
        /// get the unique key
        virtual QString getKey();

        virtual quint32 getTimestamp(){return timestamp;}

        static void resetKeyCnt(){keycnt = 0;}

    protected:

        quint32 timestamp;

        QString name;
        QString comment;
        QPixmap iconPixmap;
        QString iconString;

        static quint32 keycnt;
    private:
        virtual void genKey();
        QString key;

};

#endif //IITEM_H

